# DOT NET CORE

Download the Hesabe Dot Net Core Kit. 

Extract the contents of the Zip file. Open the folder containing the contents of the zip file. Double Click and ope the Visual Studio solution file HesabeDotNetCore.sln.

Once the project is loaded inside the IDE, go to the HesabeConstants.cs file and change the following parameter values using the values from your admin panel.

```csharp
// Get below values from Merchant Panel, Profile section
public const string ACCESS_CODE = "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx";
public const string MERCHANT_KEY = "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx";
public const string MERCHANT_IV = "xxxxxxxxxxxxxxxx";
public const string MERCHANT_CODE = "1234567890123";
```

Change the below values as per the URLs defined in your application. 
```csharp
// This URL are defined by you to get the response from Payment Gateway
public const string RESPONSE_URL = "https://yourdomain.com/PaymentResponse";
public const string FAILURE_URL = "https://yourdomain.com/PaymentFailure";
```

Once these values are updated you are done with the configuration part. Now lets move on to the integration part of the code.

### There are 4 main components of the Code:

1) HesabeConstants.cs - here you configure your credentials. (Already detailed above)
2) HesabeApiHelper.cs - this helper makes POST API call to the checkout endpoint in order to retrieve the token
3) HesabeCrypt.cs - this is the encryption and decryption library which will encrypt your payment data before making API call or 
					before posting to the payment URL. Once you recieve the response, then this library will help you to decrypt that as well.
4) Class Models - These are the models which defines your request and response data.


Checkout Form:
```
 @using (Html.BeginForm("Index", "Home", FormMethod.Post))
{
			<p>
				Amount: <input asp-for="@Model.Amount" />
			</p>

			<p>
				Order Reference Number: <input asp-for="@Model.OrderReferenceNumber" />
			</p>

			<p>
				Payment Type: <select asp-for="@Model.PaymentType" asp-items="Html.GetEnumSelectList<HesabePaymentType>()"></select>
			</p>

			<p>
				Variable 1: <input asp-for="@Model.Variable1" />
			</p>

			<p>
				Variable 2: <input asp-for="@Model.Variable2" />
			</p>

			<p>
				Variable 3: <input asp-for="@Model.Variable3" />
			</p>

			<p>
				Variable 4: <input asp-for="@Model.Variable4" />
			</p>

			<p>
				Variable 5: <input asp-for="@Model.Variable5" />
			</p>

			<input type="submit" value="Checkout" />
}
```

Processing the Checkout Data:

```csharp
//create the instance of HesabeCrypt encryption/decryption library using the configuration values
var hesabeCrypt = new HesabeCrypt(HesabeConstants.MERCHANT_KEY, HesabeConstants.MERCHANT_IV);

//JSON serialize the checkout data
var paymentDataJsonSerialized = JsonConvert.SerializeObject(paymentData);

//Encrypt the JSON Serialized checkout data using the HesabeCrypt library
var encryptedPaymentData = hesabeCrypt.Encrypt(paymentDataJsonSerialized);

//POST the encrypted data to the Checkout API, and recieve back the response
var checkoutApiEncryptedResponse = await HesabeApiHelper.PostCheckoutApi(encryptedPaymentData);

//Decrypt the response from the Checkout API using the HesabeCrypt library
var checkoutApiDecryptedResponse = hesabeCrypt.Decrypt(checkoutApiEncryptedResponse);

//Deserialize the decrypted text into the class model
var checkoutApiResponse = JsonConvert.DeserializeObject<HesabeCheckoutResponse>(checkoutApiDecryptedResponse);

//Redirect the user to the payment page using the data from the decrypted response
return Redirect($"{HesabeConstants.PAYMENT_URL}?data={checkoutApiResponse.Response.Data}");
```


Processing Payment Success Response:

```csharp
//create the instance of HesabeCrypt encryption/decryption library using the configuration values
var hesabeCrypt = new HesabeCrypt(HesabeConstants.MERCHANT_KEY, HesabeConstants.MERCHANT_IV);

//Decrypt the response recieved in the data query string 
var dataDecrypted = hesabeCrypt.Decrypt(data);

//Trim the junk characters from the derypted response if any.
dataDecrypted = dataDecrypted.Substring(0, dataDecrypted.LastIndexOf("}") + 1);

//Desrialize the decrypted response
var paymentResponse = JsonConvert.DeserializeObject<HesabePaymentResponse>(dataDecrypted);
```


Processing Payment Failure Response:

```csharp
//create the instance of HesabeCrypt encryption/decryption library using the configuration values
var hesabeCrypt = new HesabeCrypt(HesabeConstants.MERCHANT_KEY, HesabeConstants.MERCHANT_IV);

//Decrypt the response recieved in the data query string 
var dataDecrypted = hesabeCrypt.Decrypt(data);

//Trim the junk characters from the derypted response if any.
dataDecrypted = dataDecrypted.Substring(0, dataDecrypted.LastIndexOf("}") + 1);

//Desrialize the decrypted response
var paymentResponse = JsonConvert.DeserializeObject<HesabePaymentResponse>(dataDecrypted);
```
